library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity cordic_iter is
  generic(
    N     : natural := 8;  --Ancho de la palabra
    SHIFT : natural := 1); --Desplazamiento
  port(
    clk   : in std_logic;
    rst   : in std_logic;
    en_i  : in std_logic;
    xi    : in std_logic_vector (N-1 downto 0);
    yi    : in std_logic_vector (N-1 downto 0);
    zi    : in std_logic_vector (N-1 downto 0);
    ci    : in std_logic_vector (N-1 downto 0);
    dv_o  : out std_logic;
    xip1  : out std_logic_vector (N-1 downto 0);
    yip1  : out std_logic_vector (N-1 downto 0);
    zip1  : out std_logic_vector (N-1 downto 0)
  );
end entity;

architecture structural of cordic_iter is
    signal d : std_logic;
    signal x_on_sub : std_logic_vector(N-1 downto 0);
    signal x_on_add : std_logic_vector(N-1 downto 0);
    signal y_on_sub : std_logic_vector(N-1 downto 0);
    signal y_on_add : std_logic_vector(N-1 downto 0);
    signal z_on_sub : std_logic_vector(N-1 downto 0);
    signal z_on_add : std_logic_vector(N-1 downto 0);
    signal x_output : std_logic_vector(N-1 downto 0);
    signal y_output : std_logic_vector(N-1 downto 0);
    signal z_output : std_logic_vector(N-1 downto 0);
  begin
    d <= zi(N-1);
    x_on_sub <= std_logic_vector(signed(xi) - shift_right(signed(yi), SHIFT));
    x_on_add <= std_logic_vector(signed(xi) + shift_right(signed(yi), SHIFT));
    y_on_sub <= std_logic_vector(signed(yi) - shift_right(signed(xi), SHIFT));
    y_on_add <= std_logic_vector(signed(yi) + shift_right(signed(xi), SHIFT));
    z_on_sub <= std_logic_vector(unsigned(zi) - unsigned(ci));
    z_on_add <= std_logic_vector(unsigned(zi) + unsigned(ci));

    x_output <= x_on_sub when d = '0' else x_on_add;
    y_output <= y_on_add when d = '0' else y_on_sub;
    z_output <= z_on_sub when d = '0' else z_on_add;

    update_output: process(clk)
    begin
    if (rising_edge(clk)) then
      if (rst = '1') then
        dv_o <= '0';
        xip1 <= (others => '0');
        yip1 <= (others => '0');
        zip1 <= (others => '0');
      else
        dv_o <= en_i;
        xip1 <= x_output;
        yip1 <= y_output;
        zip1 <= z_output;
      end if;
    end if;
  end process;
end architecture;
