library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

--* @brief Librería de funciones básicas
package funciones is
	--* Codifica <var>v</var> como un entero sin signo de <var>N</var> bits con 10 bits fraccionarios
	function to_fixpoint (v: integer; output_width, fixed_point_position: natural) return std_logic_vector;

	type lut_t is array(natural range <>) of integer;
	function cordic_lut (output_width, fixed_point_position, lut_size: natural) return lut_t;

	function correction_coeff(output_width, fixed_point_position, lut_size: natural) return real;
end package;


package body funciones is

	function to_fixpoint (v: integer; output_width, fixed_point_position: natural) return std_logic_vector is
	begin
		return std_logic_vector(to_signed(v, output_width) sll fixed_point_position);
	end to_fixpoint;

	function cordic_lut (output_width, fixed_point_position, lut_size: natural) return lut_t is
		variable lut: lut_t(lut_size downto 0) := (others => 0);
		variable angle_in_radians: real;
		variable angle_in_degrees: real;
		variable angle_in_fixpoint: integer;
	begin
		fill_lut: for k in 0 to lut_size loop
			angle_in_radians := arctan(real(2) ** real(-k));
			angle_in_degrees := angle_in_radians / MATH_PI * real(180);
			angle_in_fixpoint := integer(angle_in_degrees * real(2 ** fixed_point_position));
			lut(k) := angle_in_fixpoint;
		end loop fill_lut;
		return lut;
	end cordic_lut;

	function correction_coeff(output_width, fixed_point_position, lut_size: natural) return real is
		variable res: real := real(1);
	begin
		calculate_coeff: for k in 0 to lut_size loop
			res := res / sqrt(real(1) + (real(2) ** real(-2 * k)));
		end loop calculate_coeff;
		return res;
	end correction_coeff;

end package body;
