library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity square_wave_gen is
	generic (
		N: natural := 16 -- Ancho de la palabra
	);
	port (
		clk:    in std_logic;
		sync:   in std_logic;
		freq:   in std_logic_vector(N-1 downto 0);
		kind:   in std_logic_vector(1 downto 0);

		sample: out std_logic_vector(N-1 downto 0);
		phase: out std_logic_vector(N-1 downto 0)
	);
end entity;

architecture structural of square_wave_gen is
	signal triangle_sample: std_logic_vector(N-1 downto 0);
	signal square_sample: std_logic_vector(N-1 downto 0);
	signal sawtooth_sample: std_logic_vector(N-1 downto 0);

	signal updated_sample: std_logic_vector(N-1 downto 0);
	signal next_counter: unsigned(N-1 downto 0);
	signal counter: unsigned(N-1 downto 0);

	constant min: std_logic_vector(N-1 downto 0) := '1' & (N-2 downto 0 => '0');
	constant max: std_logic_vector(N-1 downto 0) := '0' & (N-2 downto 0 => '1');
begin
	-- Sawtooth
	sawtooth_sample <= std_logic_vector(signed(min) + signed(counter));

	-- Triangle
	triangle_sample <= std_logic_vector(signed(min) + (signed(counter) sll 1)) when counter(N-1) = '0' else
			  std_logic_vector(signed(max) - (signed(counter) sll 1));

	-- Square
	square_sample <= std_logic_vector(min) when counter(N-1) = '0' else
			  std_logic_vector(max);

	updated_sample <= triangle_sample when kind = "11" else
		square_sample when kind = "10" else
		sawtooth_sample when kind = "01" else
		(others => '0');

	-- `clk` anda a 44100 Hz
	-- Si `clk` = 0xFFFF entonces cada clock sumo la mitad del rango
	-- Si `clk` = 0x0000 entonces cada clock sumo cero
	-- ¿¿Alcanza con regla de tres simple??
	next_counter <= (others => '0') when sync = '1' else
			counter + unsigned(freq);

	update_output: process(clk)
	begin
		if (rising_edge(clk)) then
			sample <= updated_sample(N-1 downto 0);
			phase <= std_logic_vector(counter);
			counter <= next_counter;
		end if;
	end process;

end architecture;
