library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity square_wave_gen_tb is
end entity;

architecture behavioral of square_wave_gen_tb is
	constant N: natural := 16; -- Ancho de la palabra
	constant clk_period: time := 1000 ms / 44100;
	
	signal clk: std_logic := '1';
	signal rst: std_logic := '1';
	signal en: std_logic := '0';
	signal freq: std_logic_vector(N-1 downto 0) := (others => '0');

	signal generated_sample: std_logic_vector(N-1 downto 0) := (others => '0');
	signal sample: std_logic_vector(N-1 downto 0) := (others => '0');
	signal phase: std_logic_vector(N-1 downto 0) := (others => '0');

	signal note_on: std_logic := '0';
	signal note_off: std_logic := '0';
	signal is_note_playing: std_logic := '0';
	procedure proc_do_something(
		note_freq: in natural;
		length:  time;

		signal freq_signal: out std_logic_vector(N-1 downto 0);
		signal on_signal: out std_logic;
		signal off_signal: out std_logic
	) is begin
		freq_signal <= std_logic_vector(to_unsigned(note_freq, 16));
		on_signal <= '1';
		wait for clk_period;
		on_signal <= '0';
		wait for length;
		off_signal <= '1';
		wait for clk_period;
		off_signal <= '0';
		wait until is_note_playing = '0';
	end procedure proc_do_something;
begin

	WAVE: entity work.square_wave_gen
		generic map(N => N)
		port map(
			clk => clk,
			sync => rst,
			freq => freq,
			kind => "11",
			sample => generated_sample,
			phase => phase
		);

	ADSR: entity work.adsr
		port map(
			clk => clk,
			sample => generated_sample,
			volume => std_logic_vector(to_unsigned(65535, 16)),
			note_on => note_on,
			note_off => note_off,
			is_note_playing => is_note_playing,
			rescaled_sample => sample,
			attack_increment => std_logic_vector(to_unsigned(256, 16)),
			decay_decrement => std_logic_vector(to_unsigned(16, 16)),
			sustain_level => std_logic_vector(to_unsigned(10000, 16)),
			release_decrement => std_logic_vector(to_unsigned(64, 16))
		);

	CLK_PROC: process
	begin
		clk <= '1';
		wait for clk_period / 2;
		clk <= '0';
		wait for clk_period / 2;
	end process;

	PLAY_NOTES: process
	begin
		-- E4 A4 B4
		proc_do_something(330, 100 ms, freq, note_on, note_off);
		proc_do_something(440, 100 ms, freq, note_on, note_off);
		proc_do_something(494, 100 ms, freq, note_on, note_off);
		-- C5 C5
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		proc_do_something(523, 150 ms, freq, note_on, note_off);
		-- C5 C5
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		-- C5 C5 C5 C5
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		proc_do_something(523, 150 ms, freq, note_on, note_off);
		proc_do_something(523, 150 ms, freq, note_on, note_off);
		proc_do_something(523, 150 ms, freq, note_on, note_off);
		-- C5 C5
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		-- B4 A4 C5 A4
		proc_do_something(494, 100 ms, freq, note_on, note_off);
		proc_do_something(440, 100 ms, freq, note_on, note_off);
		proc_do_something(523, 150 ms, freq, note_on, note_off);
		proc_do_something(440, 150 ms, freq, note_on, note_off);

		-- E4 A4 B4
		proc_do_something(330, 100 ms, freq, note_on, note_off);
		proc_do_something(440, 100 ms, freq, note_on, note_off);
		proc_do_something(494, 100 ms, freq, note_on, note_off);
		-- C5 C5
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		proc_do_something(523, 150 ms, freq, note_on, note_off);
		-- C5 C5
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		-- C5 C5 C5 C5
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		proc_do_something(523, 150 ms, freq, note_on, note_off);
		proc_do_something(523, 150 ms, freq, note_on, note_off);
		proc_do_something(523, 150 ms, freq, note_on, note_off);
		-- C5 C5
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		-- B4 A4 D5
		proc_do_something(494, 100 ms, freq, note_on, note_off);
		proc_do_something(440, 150 ms, freq, note_on, note_off);
		proc_do_something(587, 300 ms, freq, note_on, note_off);

		-- D4 B4 C5
		proc_do_something(294, 100 ms, freq, note_on, note_off);
		proc_do_something(494, 100 ms, freq, note_on, note_off);
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		-- D5 D5
		proc_do_something(587, 100 ms, freq, note_on, note_off);
		proc_do_something(587, 150 ms, freq, note_on, note_off);
		-- D5 D5
		proc_do_something(587, 100 ms, freq, note_on, note_off);
		proc_do_something(587, 100 ms, freq, note_on, note_off);
		-- D5 D5 D5 D5
		proc_do_something(587, 100 ms, freq, note_on, note_off);
		proc_do_something(587, 150 ms, freq, note_on, note_off);
		proc_do_something(587, 150 ms, freq, note_on, note_off);
		proc_do_something(587, 150 ms, freq, note_on, note_off);
		-- D5 D5
		proc_do_something(587, 100 ms, freq, note_on, note_off);
		proc_do_something(587, 100 ms, freq, note_on, note_off);
		-- E5 D5 D5 C5
		proc_do_something(659, 100 ms, freq, note_on, note_off);
		proc_do_something(587, 100 ms, freq, note_on, note_off);
		proc_do_something(587, 150 ms, freq, note_on, note_off);
		proc_do_something(523, 200 ms, freq, note_on, note_off);

		-- E4 A4 B4
		proc_do_something(330, 100 ms, freq, note_on, note_off);
		proc_do_something(440, 100 ms, freq, note_on, note_off);
		proc_do_something(494, 150 ms, freq, note_on, note_off);
		-- C5 C5
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		proc_do_something(523, 150 ms, freq, note_on, note_off);
		-- C5 C5
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		-- C5 C5 B4 B4
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		proc_do_something(523, 150 ms, freq, note_on, note_off);
		proc_do_something(494, 150 ms, freq, note_on, note_off);
		proc_do_something(494, 150 ms, freq, note_on, note_off);
		-- B4 B4
		proc_do_something(494, 100 ms, freq, note_on, note_off);
		proc_do_something(494, 100 ms, freq, note_on, note_off);
		-- C5 B4 A4
		proc_do_something(523, 100 ms, freq, note_on, note_off);
		proc_do_something(494, 150 ms, freq, note_on, note_off);
		proc_do_something(440, 100 ms, freq, note_on, note_off);
		wait for 50 ms;
		proc_do_something(440, 100 ms, freq, note_on, note_off);
		wait for 10 ms;
		proc_do_something(440,  80 ms, freq, note_on, note_off);
		proc_do_something(440,  80 ms, freq, note_on, note_off);
		proc_do_something(440, 500 ms, freq, note_on, note_off);
		wait for 1000 ms;
	end process;

	STIM_PROC: process
	begin
		en <= '0';
		wait for 10 ms;
		rst <= '0';
		en <= '1';
		wait;
	end process;

end architecture;
