#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void skip_word(char** ptr) {
	char* s = *ptr;
	while (*s != '\0' && *s != ' ') s++;
	if (*s == ' ') s++;
	*ptr = s;
}

void write_str(FILE* f, const char* s) {
	fprintf(f, "%s", s);
}

void write_u32(FILE* f, uint32_t v) {
	fwrite(&v, sizeof(v), 1, f);
}

void write_s32(FILE* f, int32_t v) {
	fwrite(&v, sizeof(v), 1, f);
}

void write_u16(FILE* f, uint16_t v) {
	fwrite(&v, sizeof(v), 1, f);
}

void write_s16(FILE* f, int16_t v) {
	fwrite(&v, sizeof(v), 1, f);
}

int main(int argc, char *argv[]) {
	if (argc != 3) {
		fprintf(stderr, "Usage: %s <file.vcd> <signal-name>\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	FILE* stream = fopen(argv[1], "r");
	if (stream == NULL) {
		perror("fopen");
		exit(EXIT_FAILURE);
	}

	char* signal_name = argv[2];
	size_t signal_name_len = strlen(signal_name);

	char* line = NULL;
	size_t len = 0;
	ssize_t nread;
	int signal_width;
	char* signal_id;
	size_t signal_id_len;
	while ((nread = getline(&line, &len, stream)) != -1) {
		// Should start with $var
		if (strncmp("$var", line, 4) != 0) continue;

		char* var_line = line;
		// Skip `$var`
		skip_word(&var_line);
		// Skip `reg`
		skip_word(&var_line);
		signal_width = atoi(var_line);
		// Skip width
		skip_word(&var_line);
		char* signal_id_start = var_line;
		// Skip id
		skip_word(&var_line);
		char* signal_id_end = var_line-1 /* space */;

		// Is it the expected signal?
		if (strncmp(signal_name, var_line, signal_name_len) == 0) {
			signal_id = strndup(signal_id_start, signal_id_end - signal_id_start);
			signal_id_len = signal_id_end - signal_id_start;
			break;
		}
	}
	if (nread == -1) {
		fprintf(stderr, "Failed while looking for the signal `%s`\n", signal_name);
		return 0;
	}

	if (signal_width != 16) {
		fprintf(stderr, "Expected a signal width of 16 bits. Exiting...\n");
		exit(EXIT_FAILURE);
	}

	size_t samples_size = 0;
	size_t samples_capacity = 44100 * 5 * sizeof(uint16_t);
	uint16_t* samples = malloc(samples_capacity);
	uint16_t last_seen_sample = 0;
	while ((nread = getline(&line, &len, stream)) != -1) {
		// New time-step
		if (line[0] == '#') {
			// Let's add the sample from the previous timestep
			samples[samples_size++] = last_seen_sample;
			if (samples_capacity == samples_size * sizeof(uint16_t)) {
				samples = realloc(samples, samples_capacity * 2);
				samples_capacity *= 2;
			}
		}
		if (line[0] != 'b') continue;

		char* var_line = line + 1;
		uint16_t sample = strtoul(var_line, NULL, 2);

		// Skip value
		skip_word(&var_line);
		// Is it the expected signal?
		if (strncmp(signal_id, var_line, signal_id_len) == 0) {
			last_seen_sample = sample;
		}
	}
	if (!feof(stream)) {
		fprintf(stderr, "Failed while reading samples\n");
		return 0;
	}
	printf("signal_id is %s\n", signal_id);

	FILE* outfile = fopen("out.wav", "w");
	if (stream == NULL) {
		perror("fopen");
		exit(EXIT_FAILURE);
	}
	/* ckID */   write_str(outfile,"RIFF");
	/* cksize */ write_u32(outfile, 4 + 24 + (8 + sizeof(uint16_t) * samples_size));
		/* WAVEID */  write_str(outfile, "WAVE");
		/* ckID */    write_str(outfile, "fmt ");
		/* cksize */  write_u32(outfile, 16);
			/* wFormatTag */      write_u16(outfile, 1 /* WAVE_FORMAT_PCM */);
			/* nChannels */       write_u16(outfile, 1);
			/* nSamplesPerSec */  write_u32(outfile, 44100);
			/* nAvgBytesPerSec */ write_u32(outfile, 44100 * sizeof(uint16_t));
			/* nBlockAlign */     write_u16(outfile, sizeof(uint16_t));
			/* wBitsPerSample */  write_u16(outfile, 16);
		/* ckID */    write_str(outfile, "data");
		/* cksize */  write_u32(outfile, sizeof(uint16_t) * samples_size);
		/* samples */ fwrite(samples, samples_size * sizeof(uint16_t), 1, outfile);
	fclose(outfile);

	printf("Wrote %ld samples to out.wav\n", samples_size);

	free(line);
	free(samples);
	fclose(stream);
	exit(EXIT_SUCCESS);
}
